import { minioClient } from '@/app.ts';
import { bucketName } from '@/modules/minio.ts';
import { fileIsExist } from '@kevisual/use-config';

import fs from 'fs';
import path from 'path';
import * as tar from 'tar';
import { appsPath } from '../lib/index.ts';
import { installAppFromKey } from './manager.ts';
export type InstallAppOpts = {
  path?: string;
  key?: string;
  needInstallDeps?: boolean;
};
/**
 * 检测路径是否存在
 * @param opts
 * @returns
 */
export const appPathCheck = async (opts: InstallAppOpts) => {
  const { key } = opts;
  const directory = path.join(appsPath, key);
  if (fileIsExist(directory)) {
    return true;
  }
  return false;
};
export const installApp = async (opts: InstallAppOpts) => {
  const { key, needInstallDeps } = opts;
  const fileStream = await minioClient.getObject(bucketName, opts.path);
  const pathName = opts.path.split('/').pop();
  const directory = path.join(appsPath, key);
  if (!fileIsExist(directory)) {
    fs.mkdirSync(directory, { recursive: true });
  }
  const filePath = path.join(directory, pathName);

  const writeStream = fs.createWriteStream(filePath);
  fileStream.pipe(writeStream);

  await new Promise((resolve, reject) => {
    writeStream.on('finish', () => resolve(true));
    writeStream.on('error', reject);
  });
  // 解压 tgz文件
  const extractPath = path.join(directory);
  await tar.x({
    file: filePath,
    cwd: extractPath,
  });
  if (needInstallDeps) {
    try {
      installDeps({ appPath: directory, isProduction: true, sync: true });
    } catch (e) {
      console.log('installDeps error, [need manual install deps]', e);
    }
  }
  return installAppFromKey(key);
};
import { spawn, spawnSync } from 'child_process';

export const checkPnpm = () => {
  try {
    spawnSync('pnpm', ['--version']);
    return true;
  } catch (e) {
    return false;
  }
};

type InstallDepsOptions = {
  appPath: string;
  isProduction?: boolean;
  sync?: boolean;
};
export const installDeps = async (opts: InstallDepsOptions) => {
  const { appPath } = opts;
  const isProduction = opts.isProduction ?? true;
  const params = ['i'];
  if (isProduction) {
    params.push('--production');
  }
  console.log('installDeps', appPath, params);
  const syncSpawn = opts.sync ? spawnSync : spawn;
  if (checkPnpm()) {
    syncSpawn('pnpm', params, { cwd: appPath, stdio: 'inherit', env: process.env });
  } else {
    syncSpawn('npm', params, { cwd: appPath, stdio: 'inherit', env: process.env });
  }
};
