import { sequelize } from '../../../modules/sequelize.ts';
import { DataTypes, Model } from 'sequelize';

type FileUrlList = {
  path: string;
  etag: string;
  versionId: string;
};
export interface ResourceData {
  list: FileUrlList[];
  lastestVersion: string;
  updatedAt: string;
  [key: string]: any;
}
export const defaultData: ResourceData = {
  list: [],
  lastestVersion: '0.0.0',
  updatedAt: '',
};
export type Resource = {
  id?: string;
  name?: string;
  description?: string;
  source?: string;
  sourceId?: string;
  data?: ResourceData;
  version?: string;
  uid?: string;
};

/**
 * 资源管理
 */
export class ResourceModel extends Model {
  declare id: string;
  declare name: string;
  declare description: string;
  declare source: string;
  declare sourceId: string;
  declare data: ResourceData;
  declare version: string;
  declare uid: string;
}

ResourceModel.init(
  {
    id: {
      type: DataTypes.UUID,
      primaryKey: true,
      defaultValue: DataTypes.UUIDV4,
      comment: 'id',
    },
    name: {
      type: DataTypes.STRING, // 第一次创建之后就不能修改了，因为这个是用来做唯一标识的
      defaultValue: '',
    },
    description: {
      type: DataTypes.TEXT,
      defaultValue: '',
    },
    source: {
      type: DataTypes.STRING,
      defaultValue: '',
    },
    sourceId: {
      type: DataTypes.STRING,
      defaultValue: '',
    },
    version: {
      type: DataTypes.STRING,
      defaultValue: '0.0.0',
    },
    data: {
      type: DataTypes.JSON,
      defaultValue: {},
    },
    uid: {
      type: DataTypes.UUID,
      allowNull: true,
    },
  },
  {
    sequelize,
    tableName: 'kv_resource',
    paranoid: true,
  },
);

ResourceModel.sync({ alter: true, logging: false }).catch((e) => {
  console.error('ResourceModel sync', e);
});
